--[[
Name: GnomeWorks_BankItems
Description: Adds a BankItems column to GnomeWorks Frame
Revision: $Revision: 4 $
Author: Whitetooth
Email: hotdogee [at] gmail [dot] com
LastUpdate: $Date: 2010-10-30 17:57:37 +0000 (Sat, 30 Oct 2010) $
Credits: 
- lilsparky for GnomeWorks TradeSkill Frame
- Xinhuan for BankItems
]]
if not BankItems_Save or not BankItems_Save.Upgraded == 4 then return end

local L = {}
function L:NewLocale(locale, t)
  if locale == true or GetLocale() == locale then
    for k, v in pairs(t) do
      self[k] = (v == true and k) or v
    end
  end
end

L:NewLocale(true, {
	["BankItems"] = true,
})
L:NewLocale("zhTW", {
	["BankItems"] = "BankItems",
})

-- BankItems cache

-- Table Pool for recycling tables
local tablePool = setmetatable({}, {__mode = "kv"}) -- Weak table

-- Get a new table
local function newTable()
	local t = next(tablePool) or {}
	tablePool[t] = nil
	return t
end

-- Delete table and add to table pool
local function delTable(t)
	if type(t) == "table" then
		for k, v in pairs(t) do
			if type(v) == "table" then
				delTable(v) -- child tables get put into the pool
			end
			t[k] = nil
		end
		setmetatable(t, nil)
		t[true] = true
		t[true] = nil
		tablePool[t] = true
	end
	return nil
end

local BankItems_Cache, BankItems_SelfCache, BankItems_GuildCache

local selfPlayerRealm = strtrim(GetCVar("realmName"))
local selfPlayerName = UnitName("player").."|"..selfPlayerRealm
local selfPlayer = BankItems_Save[selfPlayerName]
local BAGNUMBERS = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 100, 101, 102, 103, -2} -- List of bag numbers used
local MAX_GUILDBANK_TABS = MAX_GUILDBANK_TABS or 6

local function BankItems_Generate_ItemCache()
	-- This function generates an item cache that contains everything all players except the current player on the current realm
	local temp
	local data = newTable()
	for key, bankPlayer in pairs(BankItems_Save) do
		local _, realm = strsplit("|", key)
		if type(bankPlayer) == "table" and selfPlayer ~= bankPlayer and realm == selfPlayerRealm and (BankItems_Save.ShowOppositeFaction or bankPlayer.faction == selfPlayer.faction) and key ~= "Behavior" and key ~= "Behavior2" then
			for num = 1, 28 do
				if bankPlayer[num] then
					--temp = strmatch(bankPlayer[num].link, "%[(.*)%]")
					temp = tonumber(strmatch(bankPlayer[num].link, "item:(%d+):"))
					if temp then
						data[temp] = data[temp] or newTable()
						data[temp][key] = data[temp][key] or newTable()
						data[temp][key].count = (data[temp][key].count or 0) + (bankPlayer[num].count or 1)
						data[temp][key].bank = (data[temp][key].bank or 0) + (bankPlayer[num].count or 1)
					end
				end
			end
			for _, bagNum in ipairs(BAGNUMBERS) do
				local theBag = bankPlayer[format("Bag%d", bagNum)]
				if theBag then
					local realSize = theBag.size
					if bagNum == 101 or bagNum == 103 then
						realSize = #theBag
					end
					for bagItem = 1, realSize do
						if theBag[bagItem] and type(theBag[bagItem].link) == "string" then
							--temp = strmatch(theBag[bagItem].link, "%[(.*)%]")
							temp = tonumber(strmatch(theBag[bagItem].link, "item:(%d+):"))
							if temp then
								data[temp] = data[temp] or newTable()
								data[temp][key] = data[temp][key] or newTable()
								data[temp][key].count = (data[temp][key].count or 0) + (theBag[bagItem].count or 1)
								if bagNum >= 0 and bagNum <= 4 or bagNum == 102 or bagNum == -2 then
									data[temp][key].inv = (data[temp][key].inv or 0) + (theBag[bagItem].count or 1)
								elseif bagNum == 100 then
									data[temp][key].equipped = (data[temp][key].equipped or 0) + (theBag[bagItem].count or 1)
									if bagItem == 20 then  -- Don't double count equipped ammo
										data[temp][key].count = data[temp][key].count - data[temp][key].equipped
									end
								elseif bagNum == 101 then
									data[temp][key].mail = (data[temp][key].mail or 0) + (theBag[bagItem].count or 1)
								elseif bagNum == 103 then
									data[temp][key].auction = (data[temp][key].auction or 0) + (theBag[bagItem].count or 1)
								else
									data[temp][key].bank = (data[temp][key].bank or 0) + (theBag[bagItem].count or 1)
								end
							end
						end
					end
				end
			end
		end
	end
	delTable(BankItems_Cache)
	BankItems_Cache = data
end

local function BankItems_Generate_SelfItemCache()
	-- This function generates an item cache with only the player's items
	local temp
	local data = newTable()
	local bankPlayer = selfPlayer
	for num = 1, 28 do
		if bankPlayer[num] then
			--temp = strmatch(bankPlayer[num].link, "%[(.*)%]")
			temp = tonumber(strmatch(bankPlayer[num].link, "item:(%d+):"))
			if temp then
				data[temp] = data[temp] or newTable()
				data[temp].count = (data[temp].count or 0) + (bankPlayer[num].count or 1)
				data[temp].bank = (data[temp].bank or 0) + (bankPlayer[num].count or 1)
			end
		end
	end
	for _, bagNum in ipairs(BAGNUMBERS) do
		local theBag = bankPlayer[format("Bag%d", bagNum)]
		if theBag then
			local realSize = theBag.size
			if bagNum == 101 or bagNum == 103 then
				realSize = #theBag
			end
			for bagItem = 1, realSize do
				if theBag[bagItem] and type(theBag[bagItem].link) == "string" then
					--temp = strmatch(theBag[bagItem].link, "%[(.*)%]")
					temp = tonumber(strmatch(theBag[bagItem].link, "item:(%d+):"))
					if temp then
						data[temp] = data[temp] or newTable()
						data[temp].count = (data[temp].count or 0) + (theBag[bagItem].count or 1)
						if bagNum >= 0 and bagNum <= 4 or bagNum == 102 or bagNum == -2 then
							data[temp].inv = (data[temp].inv or 0) + (theBag[bagItem].count or 1)
						elseif bagNum == 100 then
							data[temp].equipped = (data[temp].equipped or 0) + (theBag[bagItem].count or 1)
							if bagItem == 20 then  -- Don't double count equipped ammo
								data[temp].count = data[temp].count - data[temp].equipped
							end
						elseif bagNum == 101 then
							data[temp].mail = (data[temp].mail or 0) + (theBag[bagItem].count or 1)
						elseif bagNum == 103 then
							data[temp].auction = (data[temp].auction or 0) + (theBag[bagItem].count or 1)
						else
							data[temp].bank = (data[temp].bank or 0) + (theBag[bagItem].count or 1)
						end
					end
				end
			end
		end
	end
	delTable(BankItems_SelfCache)
	BankItems_SelfCache = data
end

local function BankItems_Generate_GuildItemCache()
	-- This function generates an item cache that contains all guilds on the current realm
	local temp
	local data = newTable()

	for key, bankPlayer in pairs(BankItems_SaveGuild) do
		local _, realm = strsplit("|", key)
		if realm == selfPlayerRealm and bankPlayer.track and (BankItems_Save.ShowOppositeFaction or bankPlayer.faction == selfPlayer.faction) then
			for tab = 1, MAX_GUILDBANK_TABS do
				if bankPlayer[tab] and bankPlayer[tab].seen then
					-- Tab exists and seen before
					local theBag = bankPlayer[tab]
					for bagItem = 1, 98 do
						if theBag[bagItem] then
							--temp = strmatch(theBag[bagItem].link, "%[(.*)%]")
							temp = tonumber(strmatch(theBag[bagItem].link, "item:(%d+):"))
							if temp then
								data[temp] = data[temp] or newTable()
								data[temp][key] = data[temp][key] or newTable()
								data[temp][key].count = (data[temp][key].count or 0) + (theBag[bagItem].count or 1)
								data[temp][key].gbank = (data[temp][key].gbank or 0) + (theBag[bagItem].count or 1)
							end
						end
					end
				end
			end
		end
	end

	delTable(BankItems_GuildCache)
	BankItems_GuildCache = data
end

local f = CreateFrame("Frame", "GnomeWorks_BankItems_CacheFrame", UIParent)

f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
f:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")
f:RegisterEvent("BANKFRAME_OPENED")
f:RegisterEvent("MAIL_SHOW")
f:RegisterEvent("MAIL_SEND_SUCCESS")
f:RegisterEvent("KNOWN_CURRENCY_TYPES_UPDATE")
f:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
f:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
f:RegisterEvent("AUCTION_HOUSE_SHOW")
f:RegisterEvent("UNIT_INVENTORY_CHANGED")
f:RegisterEvent("BAG_UPDATE")

local function BankItems_UpdateFrame_OnUpdate(self, elapsed)
	BankItems_Generate_SelfItemCache()
	self:SetScript("OnUpdate", nil)
end

f:SetScript("OnEvent", function(self, event, arg1)
	if event == "GUILDBANKBAGSLOTS_CHANGED" then
		BankItems_Generate_GuildItemCache()
	elseif event == "VARIABLES_LOADED" then
		BankItems_Generate_GuildItemCache()
		BankItems_Generate_ItemCache()
  elseif event == "PLAYER_ENTERING_WORLD" then
		BankItems_Generate_SelfItemCache()
	elseif event == "PLAYERBANKSLOTS_CHANGED" or event == "PLAYERBANKBAGSLOTS_CHANGED" then
		BankItems_Generate_SelfItemCache()
	elseif event == "BANKFRAME_OPENED" then
		BankItems_Generate_SelfItemCache()
	elseif event == "MAIL_SHOW" then
		self:RegisterEvent("MAIL_CLOSED")
		self:RegisterEvent("MAIL_INBOX_UPDATE")
	elseif event == "MAIL_INBOX_UPDATE" then
		BankItems_Generate_SelfItemCache()
	elseif event == "MAIL_CLOSED" then
		BankItems_Generate_SelfItemCache()
		self:UnregisterEvent("MAIL_CLOSED")	-- Because it can fire more than once if you walk away from mailbox
		self:UnregisterEvent("MAIL_INBOX_UPDATE")
	elseif event == "MAIL_SEND_SUCCESS" then
		BankItems_Generate_ItemCache()
	elseif event == "KNOWN_CURRENCY_TYPES_UPDATE" or event == "CURRENCY_DISPLAY_UPDATE" then
		BankItems_Generate_SelfItemCache()
	elseif event == "AUCTION_OWNED_LIST_UPDATE" or event == "AUCTION_HOUSE_SHOW" then
		BankItems_Generate_SelfItemCache()
	elseif event == "UNIT_INVENTORY_CHANGED" and arg1 == "player" then
		-- Delay updating to the next frame as multiple UNIT_INVENTORY_CHANGED events can occur in 1 frame
		self:SetScript("OnUpdate", BankItems_UpdateFrame_OnUpdate)
	elseif event == "BAG_UPDATE" then
		-- Delay updating to the next frame as multiple BAG_UPDATE events can occur in 1 frame
		self:SetScript("OnUpdate", BankItems_UpdateFrame_OnUpdate)
	end
end)


do
	local plugin

	local function Register()
		if not BankItems_Save or not BankItems_Save.Upgraded == 4 then return end
    
    local BankItemsFilterMenu = {
		}
    
    local BankItemsColumnHeader = {
			name = L["BankItems"],
			width = 57,
			headerAlign = "CENTER",
			align = "RIGHT",
			font = "GameFontHighlightSmall",
			sortCompare = function(a, b)
				return (a.BankItemsTotalCount or 0) - (b.BankItemsTotalCount or 0)
			end,
			draw = function (rowFrame, cellFrame, entry)
				if not entry.subGroup then
					cellFrame.text:SetText(entry.BankItemsSelfCount.."/"..entry.BankItemsTotalCount)
				else
					cellFrame.text:SetText("")
				end
			end,
			enabled = function()
				return GnomeWorks.tradeID ~= 53428 -- don't show for Runeforging
			end,
		}

    local function updateSkillData(scrollFrame, entry)
      local item = GnomeWorks:GetTradeSkillItemLink(entry.index)
      item = item and tonumber(item:match("item:(%d+):"))
			if not entry.subGroup then
        local totalCount = 0
        local selfCount = 0
        if BankItems_SelfCache[item] then
          selfCount = BankItems_SelfCache[item].count
          totalCount = totalCount + BankItems_SelfCache[item].count
        end
        if BankItems_Cache[item] then
          for who, counttable in pairs(BankItems_Cache[item]) do
            totalCount = totalCount + counttable.count
          end
        end
        if BankItems_GuildCache[item] then
          for who, counttable in pairs(BankItems_GuildCache[item]) do
            totalCount = totalCount + counttable.count
          end
        end
				entry.BankItemsSelfCount = selfCount
				entry.BankItemsTotalCount = totalCount
			end
		end

		local function updateReagentData(scrollFrame, entry)
			local item = entry.id
      local totalCount = 0
      local selfCount = 0
      if BankItems_SelfCache[item] then
        selfCount = BankItems_SelfCache[item].count
        totalCount = totalCount + BankItems_SelfCache[item].count
      end
      if BankItems_Cache[item] then
        for who, counttable in pairs(BankItems_Cache[item]) do
          totalCount = totalCount + counttable.count
        end
      end
      if BankItems_GuildCache[item] then
        for who, counttable in pairs(BankItems_GuildCache[item]) do
          totalCount = totalCount + counttable.count
        end
      end
      entry.BankItemsSelfCount = selfCount
      entry.BankItemsTotalCount = totalCount
		end
    
    local SkillListScrollFrame = GnomeWorks:GetSkillListScrollFrame()
    local ReagentListScrollFrame = GnomeWorks:GetReagentListScrollFrame()
    
    SkillListScrollFrame:RegisterRowUpdate(updateSkillData, plugin)
    ReagentListScrollFrame:RegisterRowUpdate(updateReagentData, plugin)
    
    SkillListScrollFrame:AddColumn(BankItemsColumnHeader, plugin)
    ReagentListScrollFrame:AddColumn(BankItemsColumnHeader, plugin)

		return true
	end


	plugin = GnomeWorks:RegisterPlugin("BankItems", Register)

end


